<?php
namespace Teknisa\CMP\Controllers;

use Zeedhi\Framework\DTO;
use Setforms2\Shared\ORM\DateTime;
use Setforms2\Shared\ORM\DateInterval;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response\Error;

class NecessidadeComprasController
{
	private $central;
    private $parasupri;
    private $parcmpsup;
    private $paramgeral;
        
	protected $crudService;
    protected $entityManager;
    protected $service;
    protected $environment;
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $service)
    {

        $this->crudService  = $crudService;
		$this->environment = $environment;
        $this->service     = $service;
	}
	
	private function getCentralCompras()
	{
		$dataset = $this->service->getCentralCompras();
		if (((is_array($dataset->getRows()) && !is_null($dataset->getRows())) ? count($dataset->getRows()) : 0) < 1) {
            throw new \Exception("emptyCentralCompras");
		}
		
        return $dataset;
	}
	
	private function getParaSupri($cdfilial)
	{
		$dataset = $this->service->getParaSupri($cdfilial);
		
		if (((is_array($dataset->getRows()) && !is_null($dataset->getRows())) ? count($dataset->getRows()) : 0) < 1) {
            throw new \Exception("emptyParaSupri");
		}
		
        return $dataset;
	}
	
	private function getParCmpSup()
	{
		$dataset = $this->service->getParCmpSup();
		if (((is_array($dataset->getRows()) && !is_null($dataset->getRows())) ? count($dataset->getRows()) : 0) < 1) {
            throw new \Exception("emptyParCmpSup");
		}
		
        return $dataset;
	}
	
	private function getParamGeral()
	{
		$dataset = $this->service->getParamGeral();
		if (((is_array($dataset->getRows()) && !is_null($dataset->getRows())) ? count($dataset->getRows()) : 0) < 1) {
            throw new \Exception("emptyParamGeral");
		}
		
        return $dataset;
	}
	
	/**
     * Retorna a soma da data final com o limite de segurança
     * 
     * @param {string} $cdfilial Filiais separadas por "_" 
     * @param {string} $cdprodini Código do produto inicial
     * @param {string} $cdprodfin Código do produto final
     * @param {string} $cdgroupfili Código do agrupamento de filial
     * @param {string} $dtfin Data final informada no filtro
     * @param {string} $nrfiltrocompra Código do filtro utilizado pela rotina de compra simplificada
     * 
     * @return {string} $dtfin + limite de segurança
     * 
    */
	private function getDataLimiteSeguranca($cdfilial, $cdprodini, $cdprodfin, $cdgroupfili, $dtfin, $nrfiltrocompra)
	{
        $data = $this->service->getCalculaDataLimiteSeguranca($cdfilial, $cdprodini, $cdprodfin, $cdgroupfili, $dtfin, $nrfiltrocompra);
        $resposta['DATSEGU'] = $data->format('d/m/Y');
        return $resposta;
    }
	
	private function verificaCardapioNaoAprvado($row, $paramgeral) {
		$idutcardpadr = $paramgeral->getRows()[0]['IDUTCARDPADR'];
		
		if($idutcardpadr == 'D' || $idutcardpadr == 'S') {
			return $this->service->getCardapioNaoAprovado($row);	
		}
		
		return [];
	}
	
	public function getParametrosNecessidadeCompra(DTO\Request\Row $request, DTO\Response $response)
	{
		try {
			$central	= $this->getCentralCompras();
			$parasupri  = $this->getParaSupri($central->getRows()[0]['CDFILCENTCMP']);
	        $parcmpsup	= $this->getParCmpSup();
	        $paramgeral	= $this->getParamGeral();
	        $cardapioNaoAprovado = $this->verificaCardapioNaoAprvado($request->getRow(), $paramgeral);
	        
	        $response->addDataSet($central);
	        $response->addDataSet($parasupri);
	        $response->addDataSet($parcmpsup);
	        $response->addDataSet($paramgeral);
	        $response->addDataSet($cardapioNaoAprovado);

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
		
	}
}